"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const vscode = require("vscode");
const child_process_1 = require("child_process");
const resultsetview_1 = require("./resultsetview");
const path = require("path");
const extensionName = "frog";
const previewUri = vscode.Uri.parse('frog-view://authority/resultset');
class SessionManager {
    constructor(context) {
        this.outputChannel = vscode.window.createOutputChannel('SQL Query Results');
        context.subscriptions.push(this.outputChannel);
        this.diagnosticCollection = vscode.languages.createDiagnosticCollection(extensionName);
        context.subscriptions.push(this.diagnosticCollection);
        this.statusBarItem = vscode.window.createStatusBarItem(vscode.StatusBarAlignment.Left);
        this.statusBarItem.text = 'Frog: No connection';
        this.statusBarItem.show();
        context.subscriptions.push(this.statusBarItem);
        this.provider = new resultsetview_1.TextDocumentContentProvider();
        let registration = vscode.workspace.registerTextDocumentContentProvider('frog-view', this.provider);
    }
    set currentConnection(c) {
        this.statusBarItem.text = c.name;
        if (this.dbSession)
            this.closeSession();
        this._currentConnection = c;
    }
    get currentConnection() {
        return this._currentConnection;
    }
    isSessionActive() {
        return this.dbSession;
    }
    closeSession() {
        if (!this.dbSession)
            return;
        this.dbSession.stdin.end();
        this.dbSession.kill();
    }
    check() {
        if (!this.currentConnection) {
            vscode.window.showWarningMessage('No connection selected');
            return false;
        }
        if (this.dbSession) {
            return true;
        }
        let extPath = vscode.extensions.getExtension("tmuerell.frog").extensionPath;
        const configuration = vscode.workspace.getConfiguration('frog');
        const javaHome = configuration.get("javaHome", "");
        let java = "java";
        if (javaHome) {
            java = path.join(javaHome, 'bin', 'java');
        }
        this.dbSession = child_process_1.spawn(java, ['-Dfile.encoding=UTF-8', '-jar', extPath + '/frog-runner.jar', this.currentConnection.host, this.currentConnection.port.toString(), this.currentConnection.schema, this.currentConnection.username, this.currentConnection.password]);
        let outputBuffer = "";
        this.dbSession.stdout.on('data', (data) => {
            outputBuffer += data.toString();
            try {
                let json = JSON.parse(outputBuffer);
                const editor = vscode.window.activeTextEditor;
                const document = editor.document;
                const diagnostics = [];
                let cd = json['columnDefinitions'];
                if (!json['rows']) {
                    if ('dbOutput' in json) {
                        this.outputChannel.append(json['dbOutput'].map((x) => `DB: ${x}\n`).join(''));
                    }
                    if (json['userErrors']) {
                        this.outputChannel.append("Errors found:\n");
                        for (let error of json['userErrors'][0]['lines']) {
                            let range = document.lineAt(editor.selection.start.line + error['line'] - 1).range;
                            let diagnostic = new vscode.Diagnostic(range, error['text'], vscode.DiagnosticSeverity.Error);
                            diagnostic.source = extensionName;
                            diagnostic.code = "code";
                            diagnostics.push(diagnostic);
                            this.outputChannel.append(error['line'] + ": " + error['text'] + "\n");
                        }
                    }
                    else if (json['exception']) {
                        this.outputChannel.append("Exception caught: " + json['exceptionMessage'] + "\n");
                    }
                    else if ("affectedRows" in json) {
                        this.outputChannel.append(json['affectedRows'] + " rows affected.\n");
                    }
                    else {
                        vscode.window.showInformationMessage('Query executed successfully.');
                        this.outputChannel.append("SQL executed successfully.\n");
                    }
                }
                else {
                    this.provider.setData(json);
                    this.provider.update(previewUri);
                    const panel = vscode.window.createWebviewPanel('frogResults', 'Result Set View', vscode.ViewColumn.Two, {});
                    panel.webview.html = this.getWebviewContent(json);
                    this.outputChannel.append("Query successful.\n");
                }
                this.diagnosticCollection.set(document.uri, diagnostics);
                this.outputChannel.append("-- end@" + this.currentConnection.name + " " + new Date() + " -------------------------------\n");
                outputBuffer = "";
                this.statusBarItem.text = this.currentConnection.name + " (Connected)";
            }
            catch (e) {
                console.log(e);
            }
            this.statusBarItem.text = this.currentConnection.name + " (Connected)";
        });
        this.dbSession.stderr.on('data', (data) => {
            this.outputChannel.show(true);
            this.outputChannel.append(data.toString());
        });
        this.dbSession.on('close', (code) => {
            console.log(`child process exited with code ${code}`);
            this.dbSession = null;
        });
        return true;
    }
    getWebviewContent(data) {
        if (!data)
            return this.errorSnippet();
        if (data['rows'].length == 0)
            return this.errorSnippet();
        const configuration = vscode.workspace.getConfiguration('frog');
        const additionalStyles = configuration.get("resultSetStyles", "");
        let styles = `<style>
                    table {
                        border-collapse: collapse;
                    }
                    table td, table th {
                        padding: 2px;
                        font-size: 11px;
                        white-space: nowrap;
                    }
                    tr:hover {
                        background-color: #aaaa66;
                    }
                    #limitBox a {
                        padding: 3px;
                    }

                    .vscode-dark table {
                        border: solid 1px white;
                    }
                    .vscode-dark table td, .vscode-dark table th {
                        border: solid 1px white;
                    }
                    .vscode-dark #limitBox a {
                        color: white;
                    }
                    .vscode-dark table td.value-null {
                        background-color: #666600;
                    }

                    .vscode-light table {
                        border: solid 1px black;
                    }
                    .vscode-light table td, .vscode-light table th {
                        border: solid 1px black;
                    }
                    .vscode-light #limitBox a {
                        color: black;
                    }
                    .vscode-light table td.value-null {
                        background-color: #cccc33;
                    }

                    ${additionalStyles}
                </style>`;
        const choosableLimits = [2, 10, 50];
        let limitBox = '<div id="limitBox"><p>Limit Results: '
            + choosableLimits.map((limit) => `<a href="${encodeURI('command:frog.setRowLimit?{"rowLimit": ' + limit + '}')}">${limit}</a>`).join('') + '</p></div>';
        let text = `<body>${styles}${limitBox}<table><tr>`;
        text += data['columnDefinitions'].map((x) => `<th>${x['name']}</th>`).join('');
        text += '</tr>';
        text += data['rows'].map((r) => '<tr>' + r['columns'].map((c) => '<td class="' + (c['value'] == null ? 'value-null' : 'value-value') + '">' + (c['value'] == null ? 'NULL' : c['value']) + '</td>').join('') + '</tr>').join('');
        text += '</table></body>';
        return text;
    }
    errorSnippet() {
        return `
            <body>
                <table>
                    <tr>
                        <td>No results found!</td>
                    </tr>
                </table>
            </body>`;
    }
    execute(text, rowLimit) {
        if (!this.check())
            return;
        this.outputChannel.append("-- begin@" + this.currentConnection.name + " " + new Date() + " -------------------------------\n");
        let input = { "sql": text, "rowLimit": rowLimit };
        this.dbSession.stdin.write(JSON.stringify(input) + "\n--- END ---\n");
        this.statusBarItem.text = this.currentConnection.name + " (Connected, Query running...)";
    }
}
exports.SessionManager = SessionManager;
//# sourceMappingURL=sessionmanager.js.map